/* $Id: tds.c,v 1.4 2005/09/09 21:00:00 ichikawa Exp $ */

#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <linux/module.h>

#include <linux/delay.h>
#include <linux/fs.h>
#include <linux/interrupt.h>
#include <linux/pci.h>
#include <linux/spinlock.h>

#include <asm/atomic.h>
#include <asm/io.h>
#include <asm/semaphore.h>
#include <asm/uaccess.h>

#include "tdsio.h"
#include "tdsreg.h"


#if !defined(lint)
static const char rcsid[]
	= "$Id: tds.c,v 1.4 2005/09/09 21:00:00 ichikawa Exp $";
#endif


/* NIHON TSUSHINKI CO, LTD. Vender ID */
#define NITSUKI_VENDERID                                0x7651          /* XXX t
emporary */
    
/* device ID */
#define NITSUKI_TIME_SYNC_DATA_SAMPLER  0x0002
    
/* device major */
#define TDS_MAJOR                       222
    

#ifndef	BITSPERBYTE
#define BITSPERBYTE	8
#endif

#define TDS_INTR_SET_PIO_MODE	(TDS_INTR_DATA_EXIST | TDS_INTR_DATA_FULL)

#define TDS_INTR_SET_DMA_MODE	(TDS_INTR_DMA_DONE | TDS_INTR_DATA_FULL)


#ifdef TDS_DEBUG

#if TDS_DEBUG > 0
#define TDS_DEBUG_DEFAULT	TDS_DEBUG
#else
#define TDS_DEBUG_DEFAULT	1
#endif

#if TDS_DEBUG < 1
#define TDS_INLINE_DEBUG	__inline
#else
#define TDS_INLINE_DEBUG	/**/
#endif

#if TDS_DEBUG < 2
#define TDS_INLINE	__inline
#else
#define TDS_INLINE	/**/
#endif

#define STATIC		/**/
#define	TDS_INFINITE	(300 * HZ)

STATIC int tds_debug_level = TDS_DEBUG_DEFAULT;

#define ASSERT(cond)	KASSERT(cond, #cond)
#define IFDBG(level,cond)	if(((level) <= tds_debug_level) && (cond))
#define DMSG(level,fmt,args...) \
	do { if((level) <= tds_debug_level) printf(fmt, ##args); } while(0)

#else	/* TDS_DEBUG */

#define TDS_DEBUG_DEFAULT	0

#define STATIC		static
#define TDS_INLINE	__inline
#define	TDS_INFINITE	0

#define ASSERT(cond)	/*none*/
#define IFDBG(level,cond)	if(0)
#define DMSG(level,fmt,args...) \
	do { } while(0)

#endif	/* TDS_DEBUG */

/* device register handling */
#define	TDS_IOPORT(off)	((uint)(dev->iobase + off))


enum tds_bool {
	tds_true = 1,
	tds_false = 0
};


enum tds_debug {
	tds_none = 0,
	tds_info = 1,
	tds_debug = 2,
	tds_more_debug = 3
};

static struct pci_device_id tds_pci_tbl[] = {
        {
	    NITSUKI_VENDERID, NITSUKI_TIME_SYNC_DATA_SAMPLER,
	    PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0
	},

	{ }
};
MODULE_DEVICE_TABLE(pci, tds_pci_tbl);

static irqreturn_t tds_intr(int, void *, struct pt_regs *);
static ssize_t tdsread(struct file *, char *, size_t, loff_t *);
static ssize_t tdswrite(struct file *, const char *, size_t, loff_t *);
static int tdsioctl(struct inode *, struct file *, unsigned int, unsigned long);
static int tdsopen(struct inode *inode, struct file *filp);
static int tdsrelease(struct inode *inode, struct file *filp);
static int __init tds_probe_one(struct pci_dev *, const struct pci_device_id *);
static void __exit tds_remove_one(struct pci_dev *);
static int __init tds_module_init(void);
static void __exit tds_module_exit(void);

static struct pci_driver pci_tds_driver = {
	.name = "tds",
	.probe = tds_probe_one,
	.remove = tds_remove_one,
	.id_table = tds_pci_tbl,
};


static unsigned int unit_number = 0;
static int major = TDS_MAJOR;
struct tds;
typedef struct tds *tds_t;

struct tds_dma_buf {
	void *virtaddr;
	dma_addr_t dmaaddr;
	size_t size;
	tds_t dev;

	size_t write_offset;
	size_t read_offset;
	unsigned flags;
};
typedef struct tds_dma_buf tds_dma_buf_t;

/* data structure for each device */
struct tds {
	struct pci_dev *pdev;

	/* *BSD likely */
	int dv_unit;			/* unit number */
	char dv_xname[8];		/* name + unit number (e.g. foo0) */

	/* bus space addresse. */
	unsigned iobase;		/* iomem. see /proc/iomem */
	int iosize;			/* region size in bytes of CSRs */
 
	unsigned int irq;		/* number of iterrupt request */

	uint8_t rev;

	int sampling_freq;
	int sampling_width;
	int sampling_channel;

	int verbose;
	int error;
	int read_mode;
	int max_fifo_overflow;
	uint8_t sampling_flag;
	uint8_t intr_flag;
	unsigned sampling_start : 1;
	unsigned sampling_done : 1;
	unsigned pps_sync_done : 1;
	unsigned time_sync_done : 1;
	unsigned time_set_done : 1;

	/* synchronization variables */
	int sync_data_count;		/* for DATA exist inerrupt */
	int sync_1pps_count;		/* for 1PPS inerrupt */
	int fifo_overhalf_count;	/* for FIFO half interrupt */
	atomic_t fifo_overflow_count;	/* for FIFO full interrupt */

	tds_dma_buf_t dmabuf[TDS_DMA_NSEG];
	size_t dmabuf_write;
	size_t dmabuf_read;
	size_t dma_buffer_size;		/* per byte */
	size_t dma_chunk_size;		/* per byte/DMA */

	struct semaphore pps_sem;
	wait_queue_head_t pps_waitq;
	wait_queue_head_t data_waitq;
	spinlock_t *sample_lock;
	spinlock_t *unit_lock;

	tds_t next_tds;
};
tds_t tdses = NULL;

static struct file_operations tds_fops = {
	.read = tdsread,
	.write = tdswrite,
	.ioctl = tdsioctl,
	.open = tdsopen,
	.release = tdsrelease,
};

static uint8_t  dma_mode;

static uint8_t tds_dbuf_clr_seq[] = { 3, 2, 0, };


inline tds_t
tds_unit_search(int unit)
{
	tds_t dev;

	spin_lock(dev->unit_lock);
	for (dev = tdses;
	    dev != NULL && dev->dv_unit != unit; dev = dev->next_tds);
	spin_unlock(dev->unit_lock);

	return (dev);
}

inline void
tds_unit_add(tds_t dev)
{

	spin_lock(dev->unit_lock);
	dev->next_tds = tdses;
	tdses = dev;
	spin_unlock(dev->unit_lock);
}

inline tds_t
tds_unit_remove(int unit)
{
	tds_t dev, dev1;

	spin_lock(dev->unit_lock);
	if (tdses == NULL) {
		spin_unlock(dev->unit_lock);
		return (NULL);
	}
	for (dev = tdses;
	    dev->next_tds != NULL && dev->next_tds->dv_unit != unit;
	    dev = dev->next_tds);
	if (dev->next_tds == NULL) {
		spin_unlock(dev->unit_lock);
		return (NULL);
	}

	dev1 = dev->next_tds;
	dev->next_tds = dev->next_tds->next_tds;
	spin_unlock(dev->unit_lock);
	return (dev1);
}


static TDS_INLINE void
tdsreg_clear_fifo(tds_t dev)
{
	uint reg = TDS_IOPORT(TDS_REG_WRITE_FIFO_CLEAR);
	int num = sizeof (tds_dbuf_clr_seq) / sizeof (uint8_t);
	int i;
	for (i = 0; i < num; i ++)
		outb(tds_dbuf_clr_seq[i], reg);
}


#ifdef	OBSOLATED
static TDS_INLINE void
tdsreg_check_signal(tds_t dev)
{
	uint reg = TDS_IOPORT(TDS_REG_WRITE_SYNCRONIZE);
	outb(TDS_SIGNAL_CHECK, reg);
}
#endif	/*OBSOLATED*/


static TDS_INLINE void
tdsreg_sync_1pps(tds_t dev)
{
	uint reg = TDS_IOPORT(TDS_REG_WRITE_SYNCRONIZE);
	outb(TDS_1PPS_SYNC, reg);
	outb(0, reg);
}


static TDS_INLINE void
tdsreg_sync_time(tds_t dev)
{
	uint reg = TDS_IOPORT(TDS_REG_WRITE_SYNCRONIZE);
	outb(TDS_TIME_SYNC, reg);
}


static TDS_INLINE void
tdsreg_set_irq(tds_t dev, uint8_t flags)
{
	uint reg = TDS_IOPORT(TDS_REG_WRITE_INTERRUPT_SET);
	IFDBG(tds_more_debug, 1)
		printk("%s: Set Interrupt (0x%02x)\n", dev->dv_xname, flags);
	outb(flags, reg);
}


static TDS_INLINE uint8_t
tdsreg_get_irq(tds_t dev)
{
	uint reg = TDS_IOPORT(TDS_REG_READ_INTERRUPT);
	uint8_t val = inb(reg);
	return (val);
}


static TDS_INLINE void
tdsreg_set_time(tds_t dev, uint32_t time)
{
	uint reg = TDS_IOPORT(TDS_REG_WRITE_TIME_SET);
	outl(time, reg);
}


static TDS_INLINE uint32_t
tdsreg_get_time(tds_t dev)
{
	uint reg = TDS_IOPORT(TDS_REG_READ_TIME);
	uint32_t val = inl(reg);
	return (val);
}


static TDS_INLINE uint8_t
tdsreg_get_year(tds_t dev)
{
	uint reg = TDS_IOPORT(TDS_REG_READ_YEAR);
	uint8_t val = inb(reg);
	return (val);
}


static TDS_INLINE void
tdsreg_set_sampling(tds_t dev, uint8_t flags)
{
	uint reg = TDS_IOPORT(TDS_REG_WRITE_SAMPLING_SET);
	outb(flags, reg);
}


static TDS_INLINE uint8_t
tdsreg_get_status(tds_t dev)
{
	uint reg = TDS_IOPORT(TDS_REG_READ_STATUS);
	uint8_t val = inb(reg);
	return (val);
}


static TDS_INLINE void
tdsreg_start_sampling(tds_t dev)
{
	uint reg = TDS_IOPORT(TDS_REG_WRITE_SAMPLING_START);
	outb(TDS_SAMPLING_START, reg);
}


static TDS_INLINE void
tdsreg_stop_sampling(tds_t dev)
{
	uint reg = TDS_IOPORT(TDS_REG_WRITE_SAMPLING_START);
	outb(TDS_SAMPLING_STOP, reg);
}


static TDS_INLINE uint32_t
tdsreg_get_data(tds_t dev)
{
	uint reg = TDS_IOPORT(TDS_REG_READ_DATA);
	uint32_t val = inl(reg);
	return (val);
}


static TDS_INLINE uint8_t
tdsreg_get_input_status(tds_t dev)
{
	uint reg = TDS_IOPORT(TDS_REG_READ_INPUT);
	uint8_t val = inb(reg);
	return (val);
}


static TDS_INLINE void
tdsreg_set_dma_address(tds_t dev, uint32_t dstaddr)
{
	uint reg = TDS_IOPORT(TDS_REG_WRITE_DMA_ADDR_SET);
	outl(dstaddr, reg);
}


static TDS_INLINE void
tdsreg_set_dma_datasize(tds_t dev, uint32_t dsize)
{
	uint reg = TDS_IOPORT(TDS_REG_WRITE_DMA_SIZE_SET);
	outl(dsize, reg);
}


static TDS_INLINE uint32_t
tdsreg_get_dma_datasize(tds_t dev)
{
	uint reg = TDS_IOPORT(TDS_REG_READ_DMA_SIZE);
	uint32_t val = inl(reg);
	return (val);
}


static TDS_INLINE void
tdsreg_start_dma(tds_t dev)
{
	uint reg;

	if ((dma_mode & TDS_DMA_MODE_HARD) == 0) {
		reg = TDS_IOPORT(TDS_REG_WRITE_DMA_START);
		outb(TDS_DMA_START, reg);
	} else {
		reg = TDS_IOPORT(TDS_REG_WRITE_DMA_MODE);
		outb(dma_mode, reg);
	}
}

static TDS_INLINE void
tdsreg_stop_dma(tds_t dev)
{
	uint reg;

	if ((dma_mode & TDS_DMA_MODE_HARD) == 0) {
		reg = TDS_IOPORT(TDS_REG_WRITE_DMA_START);
		outb(TDS_DMA_STOP, reg);
	} else {
		reg = TDS_IOPORT(TDS_REG_WRITE_DMA_MODE);
		outb(TDS_DMA_MODE_SOFT, reg);
		dma_mode = TDS_DMA_MODE_SOFT;
	}
}


static TDS_INLINE uint8_t
tdsreg_get_dma_status(tds_t dev)
{
	uint reg = TDS_IOPORT(TDS_REG_READ_DMA_STATUS);
	return (inb(reg));
}


static TDS_INLINE void
tdsreg_set_dma_mode(tds_t dev, uint8_t mode)
{

	dma_mode = mode;
}


static TDS_INLINE uint8_t
tdsreg_get_dma_mode(tds_t dev)
{
	uint reg = TDS_IOPORT(TDS_REG_READ_DMA_MODE);
	return (inb(reg));
}


static TDS_INLINE uint32_t
tdsreg_get_write_counter(tds_t dev)
{
	uint reg = TDS_IOPORT(TDS_REG_READ_WRITE_COUNTER);
	uint32_t val  = inl(reg);
	return (val);
}


static TDS_INLINE uint32_t
tdsreg_get_read_counter(tds_t dev)
{
	uint reg = TDS_IOPORT(TDS_REG_READ_READ_COUNTER);
	uint32_t val  = inl(reg);
	return (val);
}


static TDS_INLINE void
tdsreg_clear_write_counter(tds_t dev)
{
	uint reg = TDS_IOPORT(TDS_REG_WRITE_CLEAR_WRITE_COUNTER);
	outb(TDS_FIFO_COUNTER_CLEAR, reg);
}


static TDS_INLINE void
tdsreg_clear_read_counter(tds_t dev)
{
	uint reg = TDS_IOPORT(TDS_REG_WRITE_CLEAR_READ_COUNTER);
	outb(TDS_FIFO_COUNTER_CLEAR, reg);
}


/* support functions */

static int
tds_wait_next_1pps(tds_t dev)
{
	int c, rv = 0;

	c = dev->sync_1pps_count;
	if (down_interruptible(&dev->pps_sem))
		rv = -ERESTARTSYS;

	if (rv != 0 && dev->sync_1pps_count == c) {
		dev->intr_flag |= TDS_INTR_1PPS;
		tdsreg_set_irq(dev, dev->intr_flag);
		rv = wait_event_interruptible(dev->pps_waitq,
		    dev->sync_1pps_count != c);
		dev->intr_flag &= ~TDS_INTR_1PPS;
		tdsreg_set_irq(dev, dev->intr_flag);
		if (rv != 0) {
			printk("%s: wait_event_interruptible() failed "
			    "(wait for 1pps)\n", dev->dv_xname);
			dev->error = TDS_ERROR_UNKNOWN;
		}
		if (signal_pending(current))
			rv = -ERESTARTSYS;
	}

	up(&dev->pps_sem);
	return (rv);
}

static TDS_INLINE void
tds_issue_dma(tds_t dev)
{
	int idx = dev->dmabuf_write;
	tds_dma_buf_t *buf = &(dev->dmabuf[idx]);

	tdsreg_set_dma_mode(dev, TDS_DMA_MODE_HARD | TDS_DMA_MODE_RELATIVE);
	tdsreg_set_dma_datasize(dev, dev->dma_chunk_size / sizeof (uint32_t));
	tdsreg_set_dma_address(dev, (uint32_t)buf->dmaaddr);
	tdsreg_start_dma(dev);
	printk("%s: DMA start (%d word) with buffer[%d] phys 0x%lx\n",
	    dev->dv_xname, dev->dma_chunk_size / sizeof (uint32_t),
	    idx, (unsigned long)buf->dmaaddr);
}

static TDS_INLINE int
tds_check_extension_card(tds_t dev)
{
	uint8_t stat;
	int rv;

	stat = tdsreg_get_input_status(dev);
	if (stat & TDS_EXTCARD_EXIST)
		rv = tds_true;
	else
		rv = tds_false;
	return (rv);
}


static TDS_INLINE void
tds_dma_buffer_init(tds_t dev)
{
	int i;

	for (i = 0; i < TDS_DMA_NSEG; i ++) {
		tds_dma_buf_t *buf = &dev->dmabuf[i];
		buf->write_offset = 0;
		buf->read_offset = 0;
		buf->flags = 0;
#ifdef TDS_DEBUG
		memset(buf->virtaddr, 0, dev->dma_buffer_size);
#endif
	}
	dev->dmabuf_write = 0;
	dev->dmabuf_read = 0;

	tds_issue_dma(dev);
}


static TDS_INLINE void
tds_calc_dma_chunk_size(tds_t dev)
{
#if (TDS_DMA_USE_CHUNK==0)
	size_t size, chunk;
	int bps;

	/* bps: bits per sec */
	bps = dev->sampling_freq * dev->sampling_width * dev->sampling_channel;

	/* we use chunk size 32KWord per 1Mbps */
	size = 32 * bps / 1024;

	for(chunk = TDS_DMA_MIN_CHUNK; size > chunk; chunk *= 2)
		continue;
	if (chunk > TDS_DMA_MAX_CHUNK)
		chunk = TDS_DMA_MAX_CHUNK;
	dev->dma_chunk_size = chunk * sizeof (uint32_t);
#else	/*(TDS_DMA_USE_CHUNK==0)*/
	dev->dma_chunk_size = TDS_DMA_USE_CHUNK * sizeof (uint32_t);
#endif	/*(TDS_DMA_USE_CHUNK==0)*/
	IFDBG(tds_info, 1)
		printk("%s: \tDMA chunk_size = 0x%05x(word)\n",
		    dev->dv_xname, dev->dma_chunk_size / sizeof (uint32_t));
}

static int
tds_sampling_condition(tds_t dev, uint32_t cond)
{
	struct condition_table {
		int key;
		int val;
	};

	struct condition_table freq_table[] = {
		{ TDS_SAMPLING_40KHZ,	40 * 1000	},
		{ TDS_SAMPLING_100KHZ,	100 * 1000	},
		{ TDS_SAMPLING_200KHZ,	200 * 1000	},
		{ TDS_SAMPLING_500KHZ,	500 * 1000	},
		{ TDS_SAMPLING_1MHZ,	1 * 1000000	},
		{ TDS_SAMPLING_2MHZ,	2 * 1000000	},
		{ TDS_SAMPLING_4MHZ,	4 * 1000000	},
		{ TDS_SAMPLING_8MHZ,	8 * 1000000	},
		{ TDS_SAMPLING_16MHZ,	16 * 1000000	},
		{ TDS_SAMPLING_32MHZ,	32 * 1000000	},
		{ TDS_SAMPLING_64MHZ,	64 * 1000000	},
		{ 0xff, 0xff				}
	};
	struct condition_table width_table[] = {
		{ TDS_SAMPLING_1BIT,	1	},
		{ TDS_SAMPLING_2BIT,	2	},
		{ TDS_SAMPLING_4BIT,	4	},
		{ TDS_SAMPLING_8BIT,	8	},
		{ 0xff, 0xff			}
	};
	int freq, width, channel, rv, i;

	freq = width = channel = 0;
	rv = 0;

	for (i = 0; freq_table[i].key != 0xff; i ++)
		if ((cond & TDS_MASK_SAMP_FREQ) == freq_table[i].key)
			break;
	if (freq_table[i].key != 0xff)
			freq = freq_table[i].val;
	else {
		printk("%s: unsupported freqency\n", dev->dv_xname);
		dev->error = TDS_ERROR_NOT_SUPPORTED;
		rv = -ENODEV;
	}

	for (i = 0; width_table[i].key != 0xff; i ++)
		if ((cond & TDS_MASK_SAMP_WIDTH) == width_table[i].key)
			break;
	if (freq_table[i].key != 0xff)
			width = width_table[i].val;
	else {
		printk("%s: unsupported width\n", dev->dv_xname);
		dev->error = TDS_ERROR_NOT_SUPPORTED;
		rv = -ENODEV;
	}

	switch (cond & TDS_MASK_SAMP_CH) {
	case TDS_SAMPLING_1CH:
		channel = 1;
		break;
	case TDS_SAMPLING_4CH:
		channel = 4;

		/* SET ERROR */
		if ((freq * width) > 32000000) {
			printk("%s: over 128Mbps sampling is NOT supported\n",
			    dev->dv_xname);
			dev->error = TDS_ERROR_NOT_SUPPORTED;
			rv = -ENXIO;
		}

		/* it requires extension card, check it out */
		if (tds_check_extension_card(dev) == tds_false) {
			printk("%s: extension card is NOT installed\n",
			    dev->dv_xname);
			dev->error = TDS_ERROR_EXTCARD_NOT_EXIST;
#ifdef NOT_YET
			rv = -ENXIO;
#endif
		}
		break;
	default:
		printk("%s: unsupported sample channel\n", dev->dv_xname);
		dev->error = TDS_ERROR_NOT_SUPPORTED;
		rv = -ENODEV;
		break;
	}

	if (rv == 0) { /* valid */
		dev->sampling_freq = freq;
		dev->sampling_width = width;
		dev->sampling_channel = channel;
		tds_calc_dma_chunk_size(dev);
		IFDBG(tds_info, 1)
			printk("%s: sampling condition %dHz/%dbit/%dch\n",
				dev->dv_xname, freq, width, channel);
	}
	return (rv);
}

static TDS_INLINE int
tds_dma_buf_free(tds_t dev)
{
	int index;

	for (index = TDS_DMA_NSEG - 1; index >= 0; index --) {
		tds_dma_buf_t *buf = &(dev->dmabuf[index]);
		if (buf->dmaaddr != 0) {
			pci_unmap_single(dev->pdev,
			    buf->dmaaddr, buf->size, PCI_DMA_FROMDEVICE);
			buf->dmaaddr = 0;
		}
		if (buf->virtaddr != NULL) {
			free_pages(
			    (unsigned long)buf->virtaddr, get_order(buf->size));
			buf->virtaddr = NULL;
		}
	}

	return (0);
}

static TDS_INLINE int
tds_dma_buf_alloc(tds_t dev)
{
	int fail_count;
	const int fail_max = 3;
	int index;

	for (index = 0; index < TDS_DMA_NSEG; index ++)
		dev->dmabuf[index].dev = dev;

	fail_count = 0;
	index = 0;
	while ((index < TDS_DMA_NSEG) && (fail_count < fail_max)) {
		tds_dma_buf_t *buf = &(dev->dmabuf[index]);
		if (buf->virtaddr != NULL) {
			index ++;
			continue;
		}
		buf->size = dev->dma_buffer_size * 2;
		buf->virtaddr =
		    (void *)__get_free_pages(GFP_KERNEL, get_order(buf->size));
		if (buf->virtaddr == NULL) {
			printk("%s: __get_free_pages() failure.\n",
			    dev->dv_xname);
			fail_count ++;
		} else
			index ++;
	}
	if (fail_count >= fail_max) {
		printk("%s: __get_free_pages() too many failure\n",
		    dev->dv_xname);
		tds_dma_buf_free(dev);
		return (-ENXIO);
	}

	fail_count = 0;
	index = 0;
	while ((index < TDS_DMA_NSEG) && (fail_count < fail_max)) {
		tds_dma_buf_t *buf = &(dev->dmabuf[index]);
		if (buf->dmaaddr != 0) {
			index ++;
			continue;
		}
		if (buf->virtaddr != NULL) {
			buf->dmaaddr =
			    pci_map_single(dev->pdev,
			    buf->virtaddr, buf->size, PCI_DMA_FROMDEVICE);
			if (buf->dmaaddr == 0) {
				printk("%s: pci_map_single() failure.\n",
				    dev->dv_xname);
				buf->dmaaddr = 0;
				fail_count++;
			} else {
				printk("%s: VA=%lx, PA=%lx, SZ=%lx\n",
				    dev->dv_xname,
				    (unsigned long)buf->virtaddr,
				    (unsigned long)buf->dmaaddr,
				    (unsigned long)buf->size);
				index++;
			}
		} else
			printk("%s: WARNING: buf->virtaddr == 0!!\n",
			    dev->dv_xname);
	}
	if (fail_count >= fail_max) {
		printk("%s: pci_map_single() too many failure\n",
		    dev->dv_xname);
		tds_dma_buf_free(dev);
		return (-ENXIO);
	}

	return (0);
}


static TDS_INLINE int
tds_init(tds_t dev, int mode)
{
	int rv = 0;

#ifdef TDS_DEBUG
	dev->verbose = tds_debug;
#else
	dev->verbose = tds_none;
#endif
	dev->error = TDS_ERROR_NO_ERROR;
	dev->read_mode = mode;
	dev->max_fifo_overflow = TDS_MAX_FIFO_OVERFLOW_DEFAULT;
	dev->sampling_flag = TDS_SAMPLING_DEFAULT;
	dev->intr_flag = 0;
	dev->sampling_start = 0;
	dev->sampling_done = 0;
	dev->pps_sync_done = 0;
	dev->time_sync_done = 0;
	dev->time_set_done = 0;
	dev->sync_1pps_count = 0;
	dev->sync_data_count = 0;
	dev->fifo_overhalf_count = 0;
	atomic_set(&dev->fifo_overflow_count, 0);
	dev->dma_buffer_size = TDS_DMA_SIZE;
	dev->dma_chunk_size = TDS_DMA_MIN_CHUNK * sizeof (uint32_t);/*failsafe*/

	switch (dev->read_mode) {
	case TDS_READ_USE_PIO:
		rv = tds_dma_buf_free(dev);
		break;
	case TDS_READ_USE_DMA:
		rv = tds_dma_buf_alloc(dev);
		break;
	default:
		panic("%s: unknown read mode (%d)\n",
		    dev->dv_xname, dev->read_mode);
		break;
	}

	if (tds_sampling_condition(dev, dev->sampling_flag) == 0)
		tdsreg_set_sampling(dev, dev->sampling_flag);

	return (0);
}


static TDS_INLINE int
tds_get_fifo_count(tds_t dev)
{
	int wcount = tdsreg_get_write_counter(dev);
	int rcount = tdsreg_get_read_counter(dev);

	if (wcount < rcount)	wcount += 0x40000000;
	return (wcount - rcount);
}

static TDS_INLINE int
tdsio_sampling_start(tds_t dev)
{
	int rv = 0;
	uint8_t status;

	printk("%s: %s\n", dev->dv_xname, _szVer);

	/* No Stop by Time Sync (or set) */
#ifdef TDS_COMPAT_VLBI111
	if (dev->pps_sync_done == 0) {
		printk("%s: not synchronized to 1pps yet\n", dev->dv_xname);
		dev->error = TDS_ERROR_NOT_SYNC_1PPS;
		rv = -EIO;
	} else if ((dev->time_sync_done == 0) && (dev->time_set_done == 0)) {
		printk("%s: not synchronized (or set) time yet\n",
		    dev->dv_xname);
		dev->error = TDS_ERROR_NOT_SYNC_TIME;
		rv = -EIO;
	}
#else /* TDS_COMPAT_VLBI111 */
	if ((dev->time_sync_done == 0) && (dev->time_set_done == 0)) {
		printk("%s: not synchronized (or set) time yet\n",
		    dev->dv_xname);
		dev->error = TDS_ERROR_NOT_SYNC_TIME;
		rv = -EIO;
	}
#endif /* TDS_COMPAT_VLBI111 */

	status = tdsreg_get_status(dev);
	if (!(status & TDS_STAT_10MHZ_INPUT)) {
		printk("%s: 10MHz input not detected\n", dev->dv_xname);
		dev->error = TDS_ERROR_10MHZ_NOT_DETECT;
		rv = -EIO;
	} else {
		spin_lock(dev->sample_lock);
		if (dev->sampling_start == 0) {
			dev->sampling_start = 1;

			switch (dev->read_mode) {
			case TDS_READ_USE_PIO:
				dev->intr_flag &= ~TDS_INTR_SET_DMA_MODE;
				dev->intr_flag |= TDS_INTR_SET_PIO_MODE;
				tdsreg_set_irq(dev, dev->intr_flag);
				tdsreg_start_sampling(dev);
				break;

			case TDS_READ_USE_DMA:
				tds_dma_buffer_init(dev);

				/* Intr Reg. Clear */
				tdsreg_get_irq(dev);

				dev->intr_flag &= ~TDS_INTR_SET_PIO_MODE;
				dev->intr_flag |= TDS_INTR_SET_DMA_MODE;
				tdsreg_set_irq(dev, dev->intr_flag);
				tdsreg_start_sampling(dev);

				/*
				 * actual sampling will start
				 * after next 1PPS sync
				 */
				break;

			default:
				panic("%s: unknown read mode (%d)\n",
				    dev->dv_xname, dev->read_mode);
				break;
			}
		} else
			rv = ~0;
		spin_unlock(dev->sample_lock);
	}
	return (rv);
}

static TDS_INLINE int
tdsio_sampling_stop(tds_t dev)
{
	int rv = 0;

	spin_lock(dev->sample_lock);
	if (dev->sampling_start != 0) {
		dev->sampling_start = 0;

		switch(dev->read_mode) {
		case TDS_READ_USE_PIO:
			tdsreg_stop_sampling(dev);
			break;

		case TDS_READ_USE_DMA:
			tdsreg_stop_dma(dev);
			tdsreg_stop_sampling(dev);

			/*
			 * wait for next 1PPS sync intrrupt
			 * before disabling DMA
			 */
			dev->intr_flag &= ~TDS_INTR_SET_DMA_MODE;
			dev->intr_flag |= TDS_INTR_SET_PIO_MODE;
			tds_wait_next_1pps(dev);
			break;

		default:
			panic("%s: unknown read mode (%d)\n",
			    dev->dv_xname, dev->read_mode);
			break;
		}
	} else
		rv = ~0;
	spin_unlock(dev->sample_lock);
	return (rv);
}


static irqreturn_t
tds_intr(int irq, void *dev_id, struct pt_regs *regs)
{
	tds_t dev = (tds_t)dev_id;
	int handled = 0;
	uint8_t intr;

	IFDBG(tds_more_debug, 1)
		printk("%s: called tds_intr()\n", dev->dv_xname);

	while((intr = (tdsreg_get_irq(dev) & dev->intr_flag)) != 0) {
		handled = 1;

		IFDBG(tds_more_debug, 1) {
			struct timeval time;
			do_gettimeofday(&time);
			printk("%s: Get interrupt (0x%02x) at %ld.%ld\n",
			    dev->dv_xname, intr, time.tv_sec, time.tv_usec);
		}
		if (intr & TDS_INTR_DATA_HALF) {
			IFDBG(tds_more_debug, 1) {
				struct timeval time;
				do_gettimeofday(&time);
				printk("%s: FIFO half full at %ld.%ld\n",
				    dev->dv_xname, time.tv_sec, time.tv_usec);
			}
			dev->fifo_overhalf_count++;
		}
		if (intr & TDS_INTR_DATA_FULL) {
			struct timeval time;

			do_gettimeofday(&time);
			printk("%s: FIFO full at %ld.%ld\n",
			    dev->dv_xname, time.tv_sec, time.tv_usec);
			atomic_inc(&dev->fifo_overflow_count);

			tdsreg_clear_fifo(dev);
			if (dev->sampling_start != 0) {
				dev->sampling_start = 0;
				tdsreg_stop_sampling(dev);
				if ((dev->intr_flag & TDS_INTR_SET_PIO_MODE) !=
				    TDS_INTR_SET_PIO_MODE) {
					dev->intr_flag &=
					    ~TDS_INTR_SET_DMA_MODE;
					dev->intr_flag |= TDS_INTR_SET_PIO_MODE;
					tdsreg_set_irq(dev, dev->intr_flag);
					wake_up_interruptible(
					    &dev->data_waitq);
				} else
					wake_up_interruptible(
					    &dev->data_waitq);
			}
		}
		if (intr & TDS_INTR_DMA_DONE) {
			int idx = dev->dmabuf_write;
			tds_dma_buf_t *buf = &(dev->dmabuf[idx]);
			uint32_t size =
			    tdsreg_get_dma_datasize(dev) * sizeof(uint32_t);

			buf->write_offset += size;
			/* prepare next DMA buffer to avoid overrrun A.S.A.P. */
			if (buf->write_offset + dev->dma_chunk_size >=
			    dev->dma_buffer_size) {
				int nextidx = (idx + 1) % TDS_DMA_NSEG;
				tds_dma_buf_t *nextbuf =
				    &(dev->dmabuf[nextidx]);
				buf->flags |= TDS_BUFFLAG_DATA_READY;
				if (nextbuf->flags == 0) {
					/*
					 * next buffer is free,
					 * available for DMA target
					 */
					tdsreg_set_dma_address(
					    dev, (uint32_t)nextbuf->dmaaddr);
					nextbuf->flags |=
					    TDS_BUFFLAG_WRITE_BUSY;
					dev->dmabuf_write = nextidx;
					tdsreg_start_dma(dev);
				} else {
					/*
					 * next buffer is NOT free,
					 * need to stop DMA
					 */
					tdsreg_stop_dma(dev);
					dev->sampling_start = 0;
					tdsreg_stop_sampling(dev);
					dev->intr_flag &=
					    ~TDS_INTR_SET_DMA_MODE;
					dev->intr_flag |= TDS_INTR_SET_PIO_MODE;
					tdsreg_set_irq(dev, dev->intr_flag);

					printk("%s: DMA buffer full (0x%08x)\n",
					    dev->dv_xname, intr);
				}
			} else {
				/* Enable next chunk of DMA. */
				tdsreg_start_dma(dev);
			}
			if (size > dev->dma_chunk_size)
				printk("%s: DMA finished words %d > %d, "
				    "at buff[%d] offset %d\n",
				    dev->dv_xname, size / sizeof(uint32_t),
				    dev->dma_chunk_size / sizeof(uint32_t),
				    idx, buf->write_offset);
			IFDBG(tds_more_debug, 1) {
				struct timeval time;
				do_gettimeofday(&time);
				printk("%s: DMA end (%d word) with "
				    "buffer[%d] at %ld.%ld\n",
				    dev->dv_xname, size / sizeof(uint32_t),
				    idx, time.tv_sec, time.tv_usec);
			}
			/* update current buf paramers available to reader */
			wake_up_interruptible(&dev->data_waitq);
		}
		if (intr & TDS_INTR_DATA_EXIST) {
			if (dev->sampling_start == 0)
				tdsreg_clear_fifo(dev);
			else {
				dev->sync_data_count++;
				wake_up_interruptible(&dev->data_waitq);
			}
		}
		if (intr & TDS_INTR_1PPS) {
			dev->sync_1pps_count ++;
			wake_up_interruptible(&dev->pps_waitq);
		}
	}
	return IRQ_RETVAL(handled);
}


static ssize_t
tdsread(struct file *filp, char *buff, size_t count, loff_t *offp)
{
	tds_t dev;
	ssize_t readlen = 0;
	size_t size;
	int rv = 0;

	dev = (tds_t)filp->private_data;
	IFDBG(tds_more_debug, 1)
		printk("%s: called read()\n", dev->dv_xname);

	if (dev->sampling_done == 0) {
		printk("%s: sampling flag is not set yet\n", dev->dv_xname);
		dev->error = TDS_ERROR_NOT_SET_SAMPLING;
		return (-ENXIO);
	}

	if (dev->sampling_start == 0) {
		printk("%s: sampling not started yet\n", dev->dv_xname);
		dev->error = TDS_ERROR_NOT_STARTED;
		return (-EIO);
	}

	if (access_ok(VERIFY_WRITE, buff, count) == 0)
		return (-EFAULT);

	switch (dev->read_mode) {
	case TDS_READ_USE_PIO:
	{
		uint32_t data;
		int c;
		uint8_t stat_flag;
		uint8_t saved_stat;

		stat_flag = tdsreg_get_status(dev);
		if (stat_flag & TDS_STAT_FIFO_EMPTY) {
			/* sampling start with 1pps signal */
			tds_wait_next_1pps(dev);
		}

		saved_stat = stat_flag;
		while (count - readlen > sizeof(uint32_t)) {
			stat_flag = tdsreg_get_status(dev);
			c = dev->sync_data_count;

			if (stat_flag != saved_stat) {
				IFDBG(tds_info, 1)
					printk("%s: board status changed "
					    "0x%02x -> 0x%02x\n", dev->dv_xname,
					    saved_stat, stat_flag);
				saved_stat = stat_flag;
			}
			if (stat_flag & TDS_STAT_FIFO_EMPTY) {
				dev->intr_flag |= TDS_INTR_DATA_EXIST;
				tdsreg_set_irq(dev, dev->intr_flag);
				IFDBG(tds_info, 1) {
					printk("%s: IRQ setting changed "
					    "0x%02x\n",
					    dev->dv_xname, dev->intr_flag);
					printk("%s: call "
					    "wait_event_interruptible() "
					    "in read()\n", dev->dv_xname);
				}
				rv = wait_event_interruptible(
				    dev->data_waitq,
				    dev->sync_data_count != c);
				if (rv != 0) {
					printk("%s: tsleep() failed "
					    "(wait for DATA exist)\n",
					    dev->dv_xname);
					dev->error = TDS_ERROR_UNKNOWN;
					break;
				}
				/* XXX */
				dev->intr_flag &= ~TDS_INTR_DATA_EXIST;
				tdsreg_set_irq(dev, dev->intr_flag);
				if (signal_pending(current)) {
					rv = -ERESTARTSYS;
					break;
				}
			}
			if (stat_flag & TDS_STAT_FIFO_OVERFLOW) {
				printk("%s: device FIFO overflow in read()\n",
				    dev->dv_xname);
				if (atomic_inc_and_test(
				    &dev->fifo_overflow_count) >=
				    dev->max_fifo_overflow) {
					printk("%s: "
					    "too many times FIFO overflow",
					    dev->dv_xname);
					dev->error = TDS_ERROR_FIFO_OVERFLOW;
					rv = -EIO;
					break;
				}
			}
			data = tdsreg_get_data(dev);
			size = sizeof (data);
			rv = __copy_to_user(buff + readlen, &data, size);
			if (rv != 0) {
				printk("%s: uiomove() returns %d\n",
				    dev->dv_xname, rv);
				dev->error = TDS_ERROR_UNKNOWN;
				break;
			}
			size -= rv;
			readlen += size;
			*offp += size;
		}
		if (rv < 0)
			return (rv);

		if (count - readlen > 0) {	/* deal fractions */
			/* XXX: */
		}
		break;
	}
	case TDS_READ_USE_DMA:
	{
		int *idx = &dev->dmabuf_read;
		while (count - readlen > 0) {
			tds_dma_buf_t *buf = &(dev->dmabuf[*idx]);
			caddr_t addr;

			/*
			 * block process, until data being available.
			 */
			/*
			 * wait for finishing DMA if no data is available.
			 */
			rv = wait_event_interruptible(
			    dev->data_waitq,
			    dev->dmabuf_read != dev->dmabuf_write ||
			    buf->read_offset != buf->write_offset);
			if (rv != 0) {
				printk("%s: wait_event_interruptible() failed "
				    "(wait for read buffer[%d])\n",
				    dev->dv_xname, *idx);
				dev->error = TDS_ERROR_UNKNOWN;
				break;
			}
			if (signal_pending(current)) {
				rv = -ERESTARTSYS;
				break;
			}

			/*
			 * copy sampling data from DMA buffer to uio area.
			 */
			__builtin_expect(
			    (buf->write_offset > buf->read_offset) != 0 ,1);
			size = min(count - readlen,
			    buf->write_offset - buf->read_offset);
			addr = ((caddr_t)buf->virtaddr) + buf->read_offset;
			rv = __copy_to_user(buff + readlen, addr, size);
			if (rv != 0) {
				printk("%s: uiomove() returns %d\n",
				    dev->dv_xname, rv);
				dev->error = TDS_ERROR_UNKNOWN;
				break;
			}
			size -= rv;
			buf->read_offset += size;
			readlen += size;
			*offp += size;

			/*
			 * increment read_index, only when all data were read.
			 */
			if (buf->read_offset == buf->write_offset &&
			    (buf->flags & TDS_BUFFLAG_DATA_READY) != 0) {
				/*
				 * All data stored in currnet buffer were read
				 */
				buf->read_offset = buf->write_offset = 0;
				buf->flags = 0;
				*idx = (*idx + 1) % TDS_DMA_NSEG;
			}
		}
		if (rv < 0)
			return (rv);

		break;
	}
	default:
		panic("%s: unknown read mode (%d)\n",
		    dev->dv_xname, dev->read_mode);
		break;
	}

	return (readlen);
}

static ssize_t
tdswrite(struct file *filp, const char *buff, size_t count, loff_t *offp)
{
	tds_t dev;

	dev = (tds_t)filp->private_data;
	if (dev == NULL)
		return (-ENXIO);

	IFDBG(tds_more_debug, 1)
		printk("%s: called write()\n", dev->dv_xname);

	printk("%s: write operation not supported\n", dev->dv_xname);
	dev->error = TDS_ERROR_NOT_SUPPORTED;
	return (-ENODEV);
}

static int
tdsioctl(
    struct inode *inode, struct file *filp, unsigned int cmd, unsigned long arg)
{
        tds_t dev;
	int rv = 0;

        dev = (tds_t)filp->private_data;

	IFDBG(tds_more_debug, 1)
		printk("%s: called ioctl(): command 0x%08x\n",
		    dev->dv_xname, (int)cmd);

	switch (cmd) {
	case TDSIO_INIT:
		if (dev->sampling_start != 0) {
			printk("%s: "
			    "can't initialize while sampling in progress\n",
			    dev->dv_xname);
			dev->error = TDS_ERROR_ALREADY_STARTED;
			return (-EBUSY);
		}
		tds_init(dev, TDS_READ_DEFAULT);
		return (0);

	case TDSIO_BUFFER_CLEAR:
		if (dev->sampling_start != 0) {
			printk("%s: "
			    "can't clear FIFO while sampling in progress\n",
			    dev->dv_xname);
			dev->error = TDS_ERROR_ALREADY_STARTED;
			return (-EBUSY);
		}
		tdsreg_clear_fifo(dev);
		IFDBG(tds_info, 1)
			printk("%s: cleared device FIFO\n", dev->dv_xname);
		return (0);

	case TDSIO_SYNC_1PPS:
	{
		int count = 0;
		for (count = 0; count < 2; count++) {
			uint8_t status = tdsreg_get_status(dev);
			if (status & TDS_STAT_1PPS_INPUT) {
				tdsreg_sync_1pps(dev);
				dev->pps_sync_done = 1;
				IFDBG(tds_info, 1)
					printk("%s: synchronized to 1pps\n",
					    dev->dv_xname);
				return (0);
			}
			/* XXX: one more try */
			rv = tds_wait_next_1pps(dev);
			if (rv != 0)
				break;
		}
		printk("%s: 1pps input not detected\n", dev->dv_xname);
		dev->error = TDS_ERROR_1PPS_NOT_DETECT;
		return (-EIO);
	}
	case TDSIO_SYNC_TIME:
	{
		int count = 0;
		while (1) {
			uint8_t status;
			count ++;
			status = tdsreg_get_status(dev);
			if (status & TDS_STAT_TIME_INPUT) {
				tdsreg_sync_time(dev);
				rv = tds_wait_next_1pps(dev);
				dev->time_sync_done = 1;
				IFDBG(tds_info, 1)
					printk("%s: synchronized to time\n",
					    dev->dv_xname);
				break;
			} else {
				if (count > 2) {
					printk("%s: time input not detected, "
					    "current status 0x%08x\n",
					    dev->dv_xname, status);
					dev->error = TDS_ERROR_TIME_NOT_DETECT;
					rv = -EIO;
					break;
				} else {
					/* XXX: one more try */
					rv = tds_wait_next_1pps(dev);
					if (rv != 0)
						break;
					continue;
				}
			}
		}
		break;
	}
	case TDSIO_SET_INTERRUPT:	/* OBSOLETE */
	case TDSIO_GET_INTERRUPT:	/* OBSOLETE */
		break;	/* nothing to do */

	case TDSIO_SET_TIME:
	{
		uint32_t *time = (uint32_t *)arg;
		tdsreg_set_time(dev, *time);
		dev->time_set_done = 1;
		IFDBG(tds_info, 1)
			printk("%s: set time 0x%08x\n", dev->dv_xname, *time);
		break;
	}
	case TDSIO_GET_TIME:
	{
		uint32_t *time = (uint32_t *)arg;
		*time = tdsreg_get_time(dev);
		break;
	}
	case TDSIO_GET_YEAR:
	{
		uint8_t *year = (uint8_t *)arg;
		*year = tdsreg_get_year(dev);
		break;
	}
	case TDSIO_SET_SAMPLING:
	{
		uint32_t *flags = (uint32_t *)arg;
		rv = tds_sampling_condition(dev, *flags);
		if (rv != 0)
			break;
		tdsreg_set_sampling(dev, *flags);
		dev->sampling_flag = *flags;
		dev->sampling_done = 1;
		break;
	}
	case TDSIO_GET_SAMPLING:
	{
		uint32_t *flags = (uint32_t *)arg;
		if (dev->sampling_done == 0) {
			printk("%s: not set sampling yet\n", dev->dv_xname);
			dev->error = TDS_ERROR_NOT_SET_SAMPLING;
			rv = -ENXIO;
		} else
			*flags = dev->sampling_flag;
		break;
	}
	case TDSIO_GET_STATUS:
	{
		uint32_t *stat = (uint32_t *)arg;
		uint8_t tmp;
		tmp = tdsreg_get_status(dev);
		*stat = (uint32_t)tmp;
		break;
	}
	case TDSIO_SAMPLING_START:
		if ((rv = tdsio_sampling_start(dev)) != 0) {
			printk("%s: sampling already started\n", dev->dv_xname);
			dev->error = TDS_ERROR_ALREADY_STARTED;
		}
		break;

	case TDSIO_SAMPLING_STOP:
		if ((rv = tdsio_sampling_stop(dev)) != 0) {
			printk("%s: sampling not started yet\n", dev->dv_xname);
			dev->error = TDS_ERROR_NOT_STARTED;
		}
		break;

	case TDSIO_GET_ERROR:
	{
		int *error = (int *)arg;
		*error = dev->error;
		break;
	}
	case TDSIO_SET_VERBOSE:
	{
		int *verb = (int *)arg;
		dev->verbose = *verb;
		break;
	}
	case TDSIO_SET_MODE:	/* XXX: obsolete */
		/* nothing to do */
		break;

	case TDSIO_GET_MODE:	/* XXX: obsolete */
		/* nothing to do */
		break;

	case TDSIO_SET_ALLOW_OVERFLOW:
	{
		int *num = (int *)arg;
		dev->max_fifo_overflow = *num;
		break;
	}
	case TDSIO_GET_OVERFLOW_COUNT:
	{
		int *num = (int *)arg;
		*num = atomic_read(&dev->fifo_overflow_count);
		break;
	}
	case TDSIO_SET_READ_MODE:
	{
		if (dev->sampling_start == 0) {
			int *mode = (int *)arg;
			switch (*mode) {
			case TDS_READ_USE_PIO:
				dev->read_mode = *mode;
				rv = tds_dma_buf_free(dev);
				break;
			case TDS_READ_USE_DMA:
				dev->read_mode = *mode;
				rv = tds_dma_buf_alloc(dev);
				break;
			default:
				printk("%s: unknown read mode (%08x)\n",
				    dev->dv_xname, (int)*mode);
				dev->error = TDS_ERROR_NOT_SUPPORTED;
				rv = -ENOTTY;
				break;
			}
		} else {
			printk("%s: "
			    "can't set read mode while sampling in progress\n",
			    dev->dv_xname);
			dev->error = TDS_ERROR_READ_BUSY;
			rv = -EBUSY;
		}
		break;
	}
	case TDSIO_GET_READ_MODE:
	{
		int *mode = (int *)arg;
		*mode = dev->read_mode;
		break;
	}
	case TDSIO_GET_INPUT_STATUS:
	{
		uint8_t input;
		int *status = (int *)arg;
		*status = 0;
		input = tdsreg_get_input_status(dev);
		*status += input;
		break;
	}
	case TDSIO_GET_DMA_STATUS: {
		uint8_t val;
		int *status = (int *)arg;
		*status = 0;
		val = tdsreg_get_dma_status(dev);
		*status += val;
		break;
	}
	default:
		printk("%s: unknown ioctl() command (%08x)\n",
		    dev->dv_xname, (int)cmd);
		dev->error = TDS_ERROR_NOT_SUPPORTED;
		rv = -ENOTTY;
		break;
	}

	return (rv);
}

static int
tdsopen(struct inode *inode, struct file *filp)
{
	tds_t dev;
	int minor;

	minor = iminor(inode);
	dev = tds_unit_search(minor);

	IFDBG(tds_more_debug, 1)
		printk("%s: called open()\n", dev->dv_xname);

	if (filp->private_data != NULL) {
		printk(KERN_ERR "%s: aleady open", dev->dv_xname);
#if 1
		return (-EBUSY);
#endif
	} else {
		filp->private_data = dev;
		atomic_set(&dev->fifo_overflow_count, 0);
	}

	return (0);
}

static int
tdsrelease(struct inode *inode, struct file *filp)
{
	tds_t dev;

	dev = (tds_t)filp->private_data;
	IFDBG(tds_more_debug, 1)
		printk("%s: called close()\n", dev->dv_xname);

	(void)tdsio_sampling_stop(dev);

	filp->private_data = NULL;

	return (0);
}


static int __devinit
tds_probe_one(struct pci_dev *pdev, const struct pci_device_id *id)
{
	tds_t dev;
	int status = 0;

	if ((dev = kmalloc(sizeof (struct tds), GFP_KERNEL)) == NULL) {
		status = -ENOMEM;
		goto fail0;
	}
	memset(dev, 0x00, sizeof (*dev));

	dev->pdev = pdev;
	dev->dv_unit = unit_number ++;
	snprintf(dev->dv_xname, sizeof (dev->dv_xname),
	    "%s%d", pci_tds_driver.name, dev->dv_unit);
	dev->iobase = pci_resource_start(pdev, /*Base Address*/ 0);
	dev->iosize = pci_resource_len(pdev, /*Base Address*/ 0);
	dev->irq = pdev->irq;
	init_MUTEX(&dev->pps_sem);
	init_waitqueue_head(&dev->pps_waitq);
	init_waitqueue_head(&dev->data_waitq);
	spin_lock_init(&dev->sample_lock);
	spin_lock_init(&dev->unit_lock);

	printk(
	    "%s:\tY-72674R1 TimeSync DataSampler at PCI:%s\n",
	    dev->dv_xname, pdev->slot_name);

	/* We request required resources. */
	if ((status = pci_request_regions(dev->pdev, "TimeSync DataSampler")) !=
	    0)
		goto fail0;
	if ((status = request_irq(
	    dev->irq, tds_intr, SA_SHIRQ, dev->dv_xname, dev)) != 0) {
		printk(KERN_ERR "%s: Unable to allocate IRQ %d\n",
		    dev->dv_xname, dev->irq);
		goto fail1;
	}

	pci_read_config_byte(pdev, PCI_REVISION_ID, &dev->rev);

	{
		pci_enable_device(dev->pdev);
		pci_set_master(dev->pdev);
	}

	if ((status = tds_init(dev, TDS_READ_DEFAULT)) < 0)
		goto fail2;

	dev->intr_flag = 0;
	tdsreg_set_irq(dev, dev->intr_flag);
	tdsreg_stop_sampling(dev);


	pci_set_drvdata(pdev, dev);

	if (dev->verbose >= tds_info) {
		uint8_t progif, subclass;

		pci_read_config_byte(pdev, PCI_CLASS_PROG, &progif);
		pci_read_config_byte(pdev, PCI_CLASS_DEVICE, &subclass);

		printk("%s:\tClass: %02xh\n", dev->dv_xname, pdev->class);
		printk("%s:\tSubClass: %02xh\n", dev->dv_xname, subclass);
		printk("%s:\tProgramming I/F: %02xh\n", dev->dv_xname, progif);
		printk("%s:\tRevision: %d\n", dev->dv_xname, dev->rev);
		printk("%s:\tBus No: %d\n", dev->dv_xname, pdev->bus->number);
		printk("%s:\tDevice No: %d\n",
		    dev->dv_xname, PCI_SLOT(pdev->devfn));
		printk("%s:\tFunction No: %d\n",
		    dev->dv_xname, PCI_FUNC(pdev->devfn));
		printk("%s:\tBase I/O Address: %08xh\n",
		    dev->dv_xname, dev->iobase);
		printk("%s:\tIRQ: %d\n", dev->dv_xname, dev->irq);
	}

	tds_unit_add(dev);

	return (0);

fail2:
	free_irq(dev->irq, dev);
fail1:
	pci_release_regions(dev->pdev);
fail0:
	return (status);
} 

static void __devexit
tds_remove_one(struct pci_dev *pdev)
{
	tds_t dev;

	dev = pci_get_drvdata(pdev);
	pci_set_drvdata(pdev, NULL);

	free_irq(dev->irq, dev);
	pci_release_regions(dev->pdev);

	tds_unit_remove(dev->dv_unit);
}


static int __init
tds_module_init(void)
{
	int status;

	/* It registers as chrdev. */
	if ((status =
	    register_chrdev(major,  pci_tds_driver.name, &tds_fops)) < 0) {  
		printk(KERN_ERR "%s: can't get major %d\n",
		    pci_tds_driver.name, major);
		return (status); 
	}
	if (major == 0)
		major = status;

	return (pci_module_init(&pci_tds_driver));
}
	
static void __exit
tds_module_exit(void)
{

	unregister_chrdev(major, pci_tds_driver.name);
	pci_unregister_driver(&pci_tds_driver);
}

module_init(tds_module_init);
module_exit(tds_module_exit);
